from unittest import TestCase
import numpy as np
import numpy.testing as nptest
import time
import matplotlib.pyplot as plt

from hyperspy.signals import Signal2D, BaseSignal
from empyer.signals.em_signal import EMSignal
import empyer as em


class TestEMSignal(TestCase):
    def setUp(self):
        d = np.random.rand(8, 9, 10,11)
        d[:, :, 6, 5] = 10
        d[:, :,  7, 3] = 10
        d[:, :, 6, 4] = 10
        d[:, :,  4, 3] = 10
        d[:, :,  2, 3] = 10
        self.s = Signal2D(data=d)

        self.ds = EMSignal(self.s)

    def test_mask_slicing(self):
        self.ds.manav[:, :].masig[:, :] = True
        np.testing.assert_array_equal(self.ds.inav[:, :].isig[:, :].data.mask,
                                      np.ones((8, 9, 10, 11), dtype=bool))

    def test_mask_slicing2(self):
        self.ds.manav[:, 1].masig[2:5, 3:10] = True
        np.testing.assert_array_equal(self.ds.inav[:, 1].isig[2:5, 3:10].data.mask,
                                      np.ones((9, 7, 3), dtype=bool))

    def test_mask_below(self):
        self.ds.mask_below(value=0.5)
        self.assertGreater(self.ds.min(axis=(0, 1, 2, 3)).data,0.5)

    def test_slice_mask_below(self):
        self.ds.manav[0:2, 0:2].mask_below(.5)
        self.assertGreater(self.ds.inav[0:2, 0:2].min(axis=(0, 1, 2, 3)).data, 0.5)
        self.assertLess(self.ds.min(axis=(0, 1, 2, 3)).data, 0.5)

    def test_slice_mask_below2(self):
        self.ds.manav[0:2, 0:2].masig[1:2, :].mask_below(.5)
        self.assertGreater(self.ds.inav[0:2, 0:2].isig[1:2, :].min(axis=(0, 1, 2, 3)).data, 0.5)
        self.assertLess(self.ds.min(axis=(0, 1, 2, 3)).data, 0.5)

    def test_mask_above(self):
        self.ds.mask_above(value=0.5)
        self.assertLess(self.ds.max(axis=(0, 1, 2, 3)).data, 0.5)

    def test_slice_mask_above(self):
        self.ds.manav[0:2, 0:2].mask_above(.5)
        self.assertLess(self.ds.inav[0:2, 0:2].max(axis=(0, 1, 2, 3)).data, 0.5)
        self.assertGreater(self.ds.max(axis=(0, 1, 2, 3)).data, 0.5)

    def test_slice_mask_above2(self):
        self.ds.manav[0:2, 0:2].masig[1:2, :].mask_above(.5)
        self.assertLess(self.ds.inav[0:2, 0:2].isig[1:2, :].max(axis=(0, 1, 2, 3)).data, 0.5)
        self.assertGreater(self.ds.max(axis=(0, 1, 2, 3)).data, 0.5)

    def test_slice_mask_where(self):
        self.ds.manav[0:2, 0:2].masig[1:5, :].mask_where(self.ds.inav[0:2, 0:2].isig[1:5, :].data == 10)
        self.assertLess(self.ds.inav[0:2, 0:2].isig[1:5, :].max(axis=(0, 1, 2, 3)).data, 1)

    def test_mask_circle_slice(self):
        self.ds.manav[0:2, 0:2].mask_circle(center=(5, 5), radius=3)
        self.assertTrue(self.ds.inav[1, 1].isig[5, 5].data.mask)
        self.assertTrue(self.ds.inav[1, 1].isig[7, 5].data.mask)
        self.assertFalse(self.ds.inav[1, 1].isig[1:2, 5].data.mask)
        self.assertFalse(self.ds.inav[1, 3].isig[5:6, 5].data.mask)

    def test_HAADF_mask(self):
        self.ds.add_hdaaf_intensities(np.ones((8, 9)), 1.4,-.76)

    def test_HAADF(self):
        self.ds.add_hdaaf_intensities(np.random.normal(size=(8, 9)), 1.5, .1)
        print(self.ds.thickness_filter())

    def test_lazy_signal(self):
        lazy = self.ds.as_lazy()
        print(lazy)





from unittest import TestCase
import numpy as np
from empyer.misc.ecm import ecm


class TestECM(TestCase):
    def setUp(self):
        self.test_series1 = np.ones(1000)
        self.test_series2 = np.ones(1000)
        every_other = np.remainder(list(range(1000)), 2) == 1
        self.test_series2[every_other] = 0

    def test_ecm(self):
        i = ecm(self.test_series1)
        self.assertListEqual(list(np.array(i, dtype=int)), list(np.ones(1000)))
        i2 = np.array(ecm(self.test_series2), dtype=int)



from unittest import TestCase
import numpy as np

from hyperspy.signals import Signal2D, BaseSignal
from empyer.signals.diffraction_signal import DiffractionSignal, LazyDiffractionSignal
import matplotlib.pyplot as plt
from empyer.misc.image import random_ellipse
import time


class TestDiffractionSignal(TestCase):
    def setUp(self):
        d = np.random.rand(10, 10, 512, 512)
        self.center = [276, 256]
        self.lengths = sorted(np.random.rand(2) * 50 + 100, reverse=True)
        self.angle = np.random.rand() * np.pi
        rand_points = random_ellipse(num_points=1000, center=self.center, foci=self.lengths, angle=self.angle)
        d[:, :, rand_points[:, 0], rand_points[:, 1]] = 10
        self.bs = BaseSignal(data=d, lazy=False)
        self.s = Signal2D(self.bs)
        self.ds = DiffractionSignal(self.s)

    def test_ellipse(self):
        self.ds.determine_ellipse()
        print("Centers: ", self.center, self.ds.metadata.Signal.Ellipticity.center)
        print("Lengths: ", self.lengths, self.ds.metadata.Signal.Ellipticity.lengths)
        print("Angle: ", self.angle, self.ds.metadata.Signal.Ellipticity.angle)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[0], self.center[0], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[1], self.center[1], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[0], max(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[1], min(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.angle, self.angle, places=1)

    def test_conversion(self):
        start = time.time()
        converted = self.ds.calculate_polar_spectrum(phase_width=720, parallel=False, inplace=False)
        stop = time.time()
        print("the Time is:", stop-start)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[0], self.center[0], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[1], self.center[1], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[0], max(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[1], min(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.angle, self.angle, places=1)
        print("the max is:", np.max(converted.sum(axis=(0, 1, 2)).data) / 2)
        self.assertLess((converted.sum(axis=(0, 1, 2)).data > max(converted.sum(axis=(0, 1, 2)).data)/2).sum(), 5)

    def test_parallel_conversion(self):
        converted = self.ds.calculate_polar_spectrum(phase_width=720,
                                                     parallel=True,
                                                     inplace=False)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[0], self.center[0], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[1], self.center[1], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[0], max(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[1], min(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.angle, self.angle, places=1)
        self.assertLess((converted.sum(axis=(0, 1)).data > 5000).sum(), 10)

    def test_conversion_and_mask(self):
        self.ds.masig[240:260, 0:256] = True
        converted = self.ds.calculate_polar_spectrum(phase_width=720,
                                                     radius=[0,200],
                                                     parallel=False,
                                                     inplace=False)
        print("Centers: ", self.center, self.ds.metadata.Signal.Ellipticity.center)
        print("Lengths: ", self.lengths, self.ds.metadata.Signal.Ellipticity.lengths)
        print("Angle: ", self.angle, self.ds.metadata.Signal.Ellipticity.angle)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[0], self.center[0], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.center[1], self.center[1], places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[0], max(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.lengths[1], min(self.lengths), places=-1)
        self.assertAlmostEqual(self.ds.metadata.Signal.Ellipticity.angle, self.angle, places=1)
        self.assertLess((converted.sum(axis=(0, 1)).data > 5000).sum(), 10)


class TestSegmentedDiffractionSignal(TestCase):
    def setUp(self):
        self.ellipse = np.random.rand(25, 200, 200)
        self.center = [100, 110]
        self.lengths = np.add(np.random.rand(25, 2)*30, 50)
        self.lengths = [sorted(l, reverse=True) for l in self.lengths]
        self.angle = np.random.rand(25)*np.pi
        rand_angles = np.random.rand(25, 200)*2*np.pi
        rand_points = [[[(np.cos(ang)*l[0]), np.sin(ang)*l[1]] for ang in rand_angle]
                       for l, rand_angle in zip(self.lengths, rand_angles)]
        rand_points = np.array([[[int(point[0]*np.cos(a)-point[1]*np.sin(a)+self.center[0]),
                                 int(point[1] * np.cos(a) + point[0] * np.sin(a)+self.center[1])]
                                for point in rand_point]for (rand_point, a) in zip(rand_points, self.angle)])
        for i, rand_point in enumerate(rand_points):
            self.ellipse[i, rand_point[:, 0], rand_point[:, 1]] = 100
        self.ellipse = np.random.poisson(self.ellipse)
        self.ellipse = np.reshape(self.ellipse, (5,5,200,200))
        self.ellipse = np.reshape(np.transpose([[[self.ellipse]*2]*2], axes=(0,3,1,4,2,5,6)),(10,10,200,200))
        print(np.shape(self.ellipse))
        self.ellipse = np.random.poisson(self.ellipse)
        self.bs = BaseSignal(data=self.ellipse, lazy=False)
        self.s = Signal2D(self.bs)
        self.ds = DiffractionSignal(self.s)

    def test_seg(self):
        ps = self.ds.calculate_polar_spectrum(segments=5, num_points=120, radius=[0, 80])
        ps.inav[0, 1].plot()
        ps.inav[1, 1].plot()
        plt.show()

    def test_lazy(self):
        lazy = self.ds.as_lazy()
        self.assertIsInstance(lazy, LazyDiffractionSignal)
        lazy.determine_ellipse()
        lazy.mask_below(10)
        lazy.manav[2:4,1].mask_below(10)
        lazy.calculate_polar_spectrum()


from unittest import TestCase
import numpy as np
import matplotlib.pyplot as plt
from hyperspy.signals import Signal2D, BaseSignal
from empyer.signals.diffraction_signal import PolarSignal


class TestPolarSignal(TestCase):
    def setUp(self):
        d = np.random.rand(10, 10, 200, 720)
        d[:, :, 50, 185] = 100
        d[:, :, 50, 365] = 100
        d[:, :, 50, 545] = 100
        d[:, :, 50, 5] = 100
        self.s = Signal2D(d)
        self.ps = PolarSignal(self.s)
        self.ps.set_axes(2,
                         name="k",
                         scale=1,
                         units='nm^-1')

    def test_autocorrelation(self):
        ac = self.ps.autocorrelation()
        self.assertGreater(ac.data[1, 1, 50, 180],45)

    def test_autocorrelation_mask(self):
        self.ps.masig[:, 0:20] = True
        self.ps.mask_above(value=40)
        ac = self.ps.autocorrelation()
        ac.plot()
        plt.show()
        self.assertLess(ac.data[1, 1, 50, 180], 45)

    def test_autocorrelation_mask(self):
        self.ps.masig[:, 0:20] = True
        self.ps.mask_above(value=40)
        ac = self.ps.autocorrelation(cut=40.0)
        ac.plot()
        plt.show()
        self.assertLess(ac.data[1, 1, 50, 180], 45)

    def test_fem(self):
        self.ps.fem(version='omega')
        self.ps.fem(version='rings')

    def test_fem_with_filter(self):
        self.ps.add_haadf_intensities(np.random.normal(size=(10, 10)), 1.5, .1)
        vari = self.ps.fem(version="omega")
        self.ps.plot()
        plt.show()
        self.ps.fem(version="rings")

    def test_lazy(self):
        print(self.ps.as_lazy())



from unittest import TestCase
import numpy as np
import matplotlib.pyplot as plt
from hyperspy.api import load
from empyer.misc.image import random_ellipse


class TestDiffractionSignal(TestCase):
    def setUp(self):
        file = ['/media/hdd/home/hdf5Files/row0col0.hdf5', '/media/hdd/home/hdf5Files/row0col1.hdf5']
        self.d = load(file, signal_type="DiffractionSignal", stack=True)
        self.d.masig[:1250, 885:1250] = True
        self.d.mask_border(50)
        self.d.mask_circle(center=(1050,900), radius=300)
        print(self.d)

    def test_ellipse1(self):
        self.d.determine_ellipse(num_points=20000, plot=True)
        plt.show()

    def test_elliptical_identification(self):
        self.d.determine_ellipse(num_points=2000)
        ellipse = self.d.metadata.Signal.Ellipticity
        points = random_ellipse(num_points=100, center=ellipse.center, angle=ellipse.angle, foci=ellipse.lengths)
        ps = self.d.calculate_polar_spectrum(phase_width=720, radius=[0,200])
        ps.inav[1,1].plot()
        plt.show()
        ac = ps.autocorrelation(cut=50)
        ac.sum(axis=(0, 1)).isig[:, :7.0].plot()
        plt.show()
        ac.get_summed_power_spectrum().isig[0:20, :7.0].plot(cmap='hot')
        plt.show()
        ps.sum(axis=(0, 1, 2)).plot()
        plt.show()

    def test_segmented_convert(self):
        ps = self.d.calculate_polar_spectrum(radius=[0,200], segments=2, phase_width=360)
        plt.show()
        ac = ps.autocorrelation()
        ac.get_summed_power_spectrum().isig[0:20, 3.0:7.0].plot()
        plt.show()

    def test_plot(self):

        self.d.inav[1, 1].plot()
        plt.imshow(self.d.inav[1, 1].data)
        self.d.determine_ellipse(num_points=1000)
        plt.show()

    def test_ellipse(self):
        self.d.determine_ellipse(num_points=500)

    def test_polar_unwrapping(self):
        self.d.determine_ellipse(num_points=1000)
        p = self.d.calculate_polar_spectrum()
        p.inav[1, 1].plot()
        plt.show()
        s = p.autocorrelation().get_summed_power_spectrum().isig[0:12, 3.5:6.0]
        s.plot()
        plt.show()

    def test_polar_unwrapping2(self):
        self.d.determine_ellipse()
        p = self.d.calculate_polar_spectrum()
        a = p.autocorrelation()
        a.get_summed_power_spectrum().isig[2, 3.25:5.5].plot()
        a2.get_summed_power_spectrum().isig[2, 3.25:5.5].plot()
        plt.show()

    def test_power(self):
        self.power.plot()
        plt.show()



#!/usr/bin/python
import os
import glob
import empyer
import argparse



# This program is designed to take a diffraction signal, convert the signal to polar coordinates and then output
# the angular correlation and power spectrum.

"""
OPTIONS:
-d = cwd :directory
"""


def correlation(signal_file):
    ds = empyer.load(signal_file, signal_type='diffraction_signal')
    ds.mask_below(300)
    file_name = os.path.splitext(signal_file)[0]
    ps = ds.calculate_polar_spectrum()
    ps.save(filename=file_name+'_polar.hdf5', overwrite=True)
    acs = ps.autocorrelation()
    acs.save(filename=file_name + '_angular.hdf5', overwrite=True)
    pow_s = acs.get_power_spectrum()
    pow_s.save(filename=file_name+'_angularPower.hdf5', overwrite=True)
    return


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-d",
                        "--directory",
                        type=str,
                        help="input directory")
    args = parser.parse_args()

    if not args.directory:
        args.directory = os.getcwd()
    if os.path.isdir(args.directory):
        files = glob.glob(args.directory+"/*.hdf5")
    else:
        files = [args.directory]
    print(files)

    for f in files:
        correlation(f)



from unittest import TestCase
import numpy as np
import hyperspy.api as hs


class TestRegisterSignal(TestCase):
    def setUp(self):
        d = np.random.rand(10, 10, 200, 720)
        d[:, :, 50, 185] = 100
        d[:, :, 50, 365] = 100
        d[:, :, 50, 545] = 100
        d[:, :, 50, 5] = 100
        self.s = hs.signals.Signal2D(d)

    def testCasting(self):
        self.s.set_signal_type("DiffractionSignal")
        self.s.determine_ellipse()
        print(hs.print_known_signal_types())



from unittest import TestCase
import matplotlib.pyplot as plt

from empyer.misc.kernels import s_g_kernel, s_g_kern_toAng,get_wavelength, sg, shape_function


class TestConvert(TestCase):

    def test_s_g_kernel(self):
        k = s_g_kernel(100, 4, 1, 200)
        a = s_g_kern_toAng(k, 4)
        k.plot()
        a.plot()
        plt.show()
        k2 = s_g_kernel(100, 4, 20, 100)
        a2 = s_g_kern_toAng(k2, 4)
        a2.plot()

        plt.show()

    def test_wavelength(self):
        self.assertAlmostEqual(2.51*10**-3, get_wavelength(200), 2)
        self.assertAlmostEqual(1.96 * 10 ** -3, get_wavelength(300), 2)

    def test_sg(self):
        print(sg(200, rotation_vector=(0, 1, 0), theta=0))

    def test_shape_function(self):
        plt.plot([shape_function(1, (i/100)+.00001)for i in range(100)])
        plt.show()




from unittest import TestCase
import numpy as np
import matplotlib.pyplot as plt
from empyer.misc.angular_correlation import angular_correlation, power_spectrum


class TestBinning(TestCase):
    def setUp(self):
        self.test1 = np.random.rand(180, 720)
        self.mask = np.zeros((180, 720), dtype=bool)
        self.mask[0:90, 20:60] = 1

    def test_angular_correlation(self):
        ac1 = angular_correlation(self.test1)

        self.assertAlmostEqual(np.mean(ac1[:, 0]), .33, places=1)
        self.assertAlmostEqual(np.mean(ac1[:, 360]), 0, places=2)

    def test_angular_correlation_mask(self):
        test = np.ma.masked_array(np.random.rand(10, 10))
        test.mask= np.zeros((10, 10), dtype=bool)
        test.mask[0:5, 2:4] = True
        ac1 = angular_correlation(test)

    def test_power_spectrum(self):
        ac1 = angular_correlation(self.test1, normalize=False)
        ac2 = angular_correlation(self.test1, self.mask)



#!/usr/bin/python
import struct
import argparse
import numpy as np
import time
from libtiff import TIFF
import os


def loadMRCfile(filepath):
    with open(filepath, mode='rb') as file:
        # getting the size of the mrc image
        file.seek(0)
        read_bytes = file.read(8)
        frame_width = struct.unpack('<i', read_bytes[0:4])[0]
        frame_height = struct.unpack('<i', read_bytes[4:8])[0]
        file.seek(256 * 4)
        #reading the images
        dataset = file.read(frame_width * frame_height * 4)
        image = np.reshape(np.frombuffer(dataset, dtype=np.float32),(frame_height,frame_width))
        return image


def loadHeader(fileName, darkref):
    print('Reading file ' + fileName)
    with open(fileName, mode='rb') as file:  # b is important -> binary
        file.seek(548)
        read_bytes = file.read(20)
        frame_width = struct.unpack('<L', read_bytes[0:4])
        frame_height = struct.unpack('<L', read_bytes[4:8])
        print('Each frame is ' + str(frame_width[0]) + ' by ' + str(frame_width[0]) + ' px.')

        file.seek(572)
        read_bytes = file.read(4)
        num_frames = struct.unpack('<i', read_bytes)
        print('Total ' + str(num_frames[0]) + ' frames collected.')

        file.seek(584)
        read_bytes = file.read(8)
        frame_rate = struct.unpack('<d', read_bytes)
        print('Image acquired at ' + str(frame_rate[0]) + ' frames per second.')

        file.seek(580)
        read_bytes = file.read(4)
        true_imagesize = struct.unpack('<L', read_bytes[0:4])

        if frame_width[0] != darkref.shape[0] or frame_height[0] != darkref.shape[1]:
            print('Norpix frame size (' + str(frame_height[0]) + ',' + str(
                frame_width[0]) + ') disagree with reference size' + str(darkref.shape))

        return num_frames[0], true_imagesize[0]


def saveFile(directory, filename, darkref, gainref, numframes, true_imagesize, savename):
    try:
        os.mkdir(directory + savename)
    except OSError as e:
        print("Directory already exists")
    with open(directory+filename, mode='rb') as file:
        print(numframes)
        s = time.time()
        for iframe in range(numframes):
            filename = "Image" + format(iframe + 1, '05') + '.tiff'
            file.seek(8192 + iframe * true_imagesize)
            read_bytes = file.read(darkref.shape[0] * darkref.shape[1] * 2)
            # loading from buffer
            frame_raw = np.reshape(np.frombuffer(read_bytes, dtype=np.uint16), darkref.shape)
            frame = (frame_raw - darkref) * gainref
            # change to 32 bit images because that is how the gain and reference are collected.
            frame = np.array(frame, dtype=np.int32)
            # saving as 32 bit tif
            tiff = TIFF.open(directory+savename+"/" + filename, mode='w')
            tiff.write_image(frame)
            tiff.close()
        st = time.time()
        print("Time for the conversion of one Image is: ", (st-s)/numframes, "seconds")


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-d",
                        "--directory",
                        type=str,
                        help="input directory")
    parser.add_argument("-f",
                        "--filename",
                        type=str,
                        help="base filename")
    parser.add_argument("-dr",
                        "--darkreference",
                        type=str,
                        help="dark reference")
    parser.add_argument("-g",
                        "--gainreference",
                        type=str,
                        help="gain reference")
    parser.add_argument("-s",
                        "--savename",
                        type=str,
                        help="saving directory name")
    args = parser.parse_args()

    if not args.directory:
        args.directory = os.getcwd()
    start = time.time()
    # loading the dark and gain references
    darkref = loadMRCfile(args.directory + args.darkreference)
    gainref = loadMRCfile(args.directory + args.gainreference)

    if darkref.shape != gainref.shape:
        print('Dark and gain reference shape disagree!')
    numframes, true_imagesize = loadHeader(args.directory+args.filename, darkref)
    # loading the images from the .seq file and then saving them as tif files
    saveFile(directory=args.directory,
             filename=args.filename,
             darkref=darkref,
             gainref=gainref,
             numframes=numframes,
             true_imagesize=true_imagesize,
             savename=args.savename)
    end = time.time()
    print('Total time elapsed: ' + str(end - start))


from unittest import TestCase
import numpy as np
from empyer.misc.ellipse_analysis import solve_ellipse, get_max_positions
from empyer.misc.cartesain_to_polar import convert
from empyer.misc.image import random_ellipse
import matplotlib.pyplot as plt


class TestConvert(TestCase):
    def setUp(self):
        self.ellipse = np.random.rand(100, 200, 200)*90
        self.center = [110, 100]
        self.lengths = np.random.rand(100, 2)*30+50
        self.lengths = [sorted(l, reverse=True) for l in self.lengths]
        self.angles = np.random.rand(100)*np.pi
        self.rand_points = np.array([random_ellipse(num_points=1000, center=self.center, foci=l, angle=a) for
                       l, a in zip(self.lengths, self.angles)])
        for i, rand_point in enumerate(self.rand_points):
            self.ellipse[i, rand_point[:, 0], rand_point[:, 1]] = 100
        self.ellipse = np.random.poisson(self.ellipse / 100 * 10 / 10 * 100)

    def test_get_max_coords(self):
        plt.imshow(self.ellipse[0,:,:])
        c = get_max_positions(image=self.ellipse[0,:,:],radius=100, num_points=500)
        plt.scatter(c[0], c[1])
        plt.show()

    def test_solve_one_ellipse(self):
        e = self.ellipse[0, :, :]
        l = self.lengths[0]
        a = self.angles[0]
        solve_ellipse(e, num_points=1000)
        print("Length: ", l, "Angle: ", a)

    def test_solve_ellipse(self):
        for e, l, a in zip(self.ellipse, self.lengths, self.angles):
            if np.abs(l[0]-l[1]) < 1:  # too close to a circle
                continue
            cen, le, an = solve_ellipse(e, num_points=75)
            print("Centers: ", self.center, cen)
            print("Lengths: ", l, le)
            print("Angle: ", a, an)
            self.assertAlmostEqual(self.center[0], cen[0], places=-1)
            self.assertAlmostEqual(self.center[1], cen[1], places=-1)
            self.assertAlmostEqual(le[0], max(l), places=-1)
            self.assertAlmostEqual(le[1], min(l), places=-1)
            self.assertAlmostEqual(a, an, places=0)

    def test_solve_ellipse_mask(self):
        e = np.ma.masked_array(self.ellipse)
        e.mask = False
        e.mask[230:275, 0:256] = True
        e.mask[0:10, 0:256] = True
        c, l, a = solve_ellipse(e, num_points=150)
        print("Centers: ", self.center, c)
        print("Lengths: ", self.lengths, l)
        print("Angle: ", self.angles, a)
        self.assertAlmostEqual(c[0], self.center[0], places=-1)
        self.assertAlmostEqual(c[1], self.center[1], places=-1)
        self.assertAlmostEqual(l[0], max(self.lengths), places=-1)
        self.assertAlmostEqual(l[1], min(self.lengths), places=-1)
        self.assertAlmostEqual(a, self.angle, places=1)



from unittest import TestCase
import numpy as np
import matplotlib.pyplot as plt
from hyperspy.signals import Signal2D
from empyer.signals.power_signal import PowerSignal


class TestPowerSignal(TestCase):
    def setUp(self):
        d = np.random.rand(10, 10, 720, 180)
        self.s = Signal2D(d)
        self.ps = PowerSignal(self.s)
        self.ps.set_axes(2,
                         name="k",
                         scale=.1,
                         units='nm^-1')

    def test_i_vs_k(self):
        self.ps.get_i_vs_k()
        self.ps.get_i_vs_k(symmetry=[6])

    def test_get_map(self):
        mapped = self.ps.get_map()
        print(mapped)
        mapped.plot()
        plt.show()
        self.ps.get_map(symmetry=10)

    def test_lazy(self):
        print(self.ps.as_lazy())



from unittest import TestCase
import numpy as np
from empyer.misc.image import bin_2d, ellipsoid_list_to_cartesian, random_ellipse
import matplotlib.pyplot as plt


class TestBinning(TestCase):
    def setUp(self):
        self.test1 = np.zeros((512, 512))
        self.test2 = np.zeros((513, 513))

    def test_binning(self):
        bin1 = bin_2d(self.test1,2)
        bin2 = bin_2d(self.test2,2)
        self.assertTupleEqual(np.shape(bin1), (256,256))
        self.assertTupleEqual(np.shape(bin2), (256,256))


class TestConversions(TestCase):
    def setUp(self):
        self.thetas = np.linspace(-1*np.pi, np.pi, num=180)
        self.radii = np.arange(1, 40)

    def test_ellipse_conversion(self):
        x, y = ellipsoid_list_to_cartesian(self.radii,
                                          self.thetas,
                                          center=[0,0],
                                          axes_lengths=[10,20],
                                          angle=np.pi/4)
        plt.scatter(x, y)
        p = random_ellipse(num_points=100, center=[0,0], foci=[20,10], angle=np.pi/4)
        plt.scatter(p[:, 0], p[:, 1])
        plt.show()

    def test_rand_ellipse(self):
        p = random_ellipse(num_points=1000, center=[0,0], foci=[100,50], angle=np.pi/2)
        plt.scatter(p[:, 0], p[:, 1])
        plt.show()






from unittest import TestCase
import numpy as np

from hyperspy.signals import Signal2D
from empyer.io import load, to_diffraction_signal, to_correlation_signal,to_polar_signal, to_power_signal


class TestIOSignal(TestCase):
    def setUp(self):
        d = np.random.rand(10, 10, 720, 180)
        self.s = Signal2D(d)
        self.ds = to_diffraction_signal(self.s)

    def test_to_diffraction_signal(self):
        ds = to_diffraction_signal(self.s)
        self.assertDictEqual(ds.axes_manager.as_dictionary(), self.s.axes_manager.as_dictionary())
        ds.mask_below(value=.5)
        ds.save(filename='temp', overwrite=True, extension='hdf5')
        ds_2 = load('temp.hdf5')
        self.assertEqual(ds.metadata.Signal.signal_type, ds_2.metadata.Signal.signal_type)
        self.assertIsInstance(ds_2.data, np.ma.masked_array)

    def test_to_polar_signal(self):
        ps = to_polar_signal(self.s)
        self.assertDictEqual(ps.axes_manager.as_dictionary(), self.s.axes_manager.as_dictionary())
        ps.save(filename='temp', overwrite=True, extension='hdf5')
        ps_2 = load('temp.hdf5')
        self.assertEqual(ps.metadata.Signal.signal_type,ps_2.metadata.Signal.signal_type)

    def test_to_correlation_signal(self):
        ds = to_correlation_signal(self.s)
        self.assertDictEqual(ds.axes_manager.as_dictionary(), self.s.axes_manager.as_dictionary())
        ds.save(filename='temp', overwrite=True, extension='hdf5')
        ds_2 = load('temp.hdf5')
        self.assertEqual(ds.metadata.Signal.signal_type, ds_2.metadata.Signal.signal_type)

    def test_to_power_signal(self):
        ps = to_power_signal(self.s)
        self.assertDictEqual(ps.axes_manager.as_dictionary(), self.s.axes_manager.as_dictionary())
        ps.save(filename='temp', overwrite=True, extension='hdf5')
        ps_2 = load('temp.hdf5')
        self.assertEqual(ps.metadata.Signal.signal_type, ps_2.metadata.Signal.signal_type)

    def test_save_and_load(self):
        self.ds.save(filename='temp', overwrite=True, extension='hdf5')
        ds_2 = load('temp.hdf5')
        self.assertEqual("diffraction_signal", ds_2.metadata.Signal.signal_type)


import numpy as np
import matplotlib.pyplot as plt
from numpy.linalg import eig, inv
from matplotlib.patches import Ellipse
from empyer.misc.cartesain_to_polar import convert


def solve_ellipse(img, interactive=False, num_points=500, plot=False, suspected_radius=None):
    """Takes a 2-d array and allows you to solve for the equivalent ellipse. Everything is done in array coord.

    Fitzgibbon, A. W., Fisher, R. B., Hill, F., & Eh, E. (1999). Direct Least Squres Fitting of Ellipses.
        IEEE Transactions on Pattern Analysis and Machine Intelligence, 21(5), 476–480.
    http://nicky.vanforeest.com/misc/fitEllipse/fitEllipse.html

    Parameters
    ----------
    img : array-like
        Image with ellipse with more intense
    interactive: bool
        Allows you to pick points for the ellipse instead of taking the top 2000 points
    plot: bool
        plots the unwrapped image as well as a super imposed ellipse
    Returns
    ----------
    center: array-like
        In cartesian coordinates or (x,y)!!!!!! arrays are in y,x
    lengths:array-like
        The 'length' in pixels of the major and minor axis
    angle:float
        In radians based on the major axis
    """

    def fit_ellipse(x,y):
        x = x[:, np.newaxis]
        y = y[:, np.newaxis]
        D = np.hstack((x * x, x * y, y * y, x, y, np.ones_like(x)))   # Design matrix [x^2, xy, y^2, x,y,1]
        S = np.dot(D.T, D)  # Scatter Matrix
        C = np.zeros([6, 6])
        C[0, 2] = C[2, 0] = 2
        C[1, 1] = -1
        E, V = eig(np.dot(inv(S), C))  # eigen decomposition to solve constrained minimization problem
        n = np.argmax(np.abs(E))   # maximum eigenvalue solution
        a = V[:, n]
        print("a is:", a)
        return a

    def ellipse_center(ellipse_parameters):
        a, b, c, d, e, f = ellipse_parameters
        denom = b**2 - (4 * a * c)
        x0 = (2 * c * d - b * e) / denom
        y0 = (2 * a * e - b * d) / denom
        return np.array([x0, y0])

    def ellipse_axis_length(ellipse_parameters):
        a, b, c, d, e, f = ellipse_parameters
        denom = b**2 - (4 * a * c)
        num1 = np.sqrt(2 * (a * e ** 2 + c * d ** 2 - b * d * e + (b ** 2 - 4 * a * c) * f) *
                       ((a+c) - np.sqrt((a - c) ** 2 + b ** 2)))
        num2 = np.sqrt(2 * (a * e ** 2 + c * d ** 2 - b * d * e + (b ** 2 - 4 * a * c) * f) *
                       ((a+c) + np.sqrt((a - c) ** 2 + b ** 2)))
        axis1 = abs(num1/denom)
        axis2 = abs(num2/denom)

        return np.sort([axis1, axis2])[::-1]

    def ellipse_angle_of_rotation(ellipse_parameters):
        a, b, c, d, e, f = ellipse_parameters
        b, d, e = b/2, d/2, e/3
        if b == 0:
            if a > c:
                return 0
            else:
                return np.pi / 2
        else:
            if a < c:
                ang = .5 * invcot((a-c)/(2*b))
                if (a<0) == (b<0): # same sign
                    ang = ang+np.pi/2
                return ang
            else:
                ang =np.pi/2 + .5 * invcot((a-c)/(2*b))
                if (a < 0) != (b < 0):
                    ang=ang-np.pi/2
                return ang


    coords = [[], []]
    if interactive:
        figure1 = plt.figure()
        ax = figure1.add_subplot(111)
        plt.imshow(i)
        plt.text(x=-50, y=-20, s="Click to add points. Click again to remove points. You must have 5 points")

        def add_point(event):
            ix, iy = event.xdata, event.ydata
            x_diff, y_diff = np.abs(np.subtract(coords[0], ix)), np.abs(np.subtract(coords[1], iy))
            truth = list(np.multiply(x_diff < 10, y_diff < 10))
            if not len(x_diff):
                coords[0].append(ix)
                coords[1].append(iy)
            elif not any(truth):
                coords[0].append(ix)
                coords[1].append(iy)
            else:
                truth = list(np.multiply(x_diff < 10, y_diff < 10)).index(True)
                coords[0].pop(truth)
                coords[1].pop(truth)
            print(coords)
            ax.clear()
            ax.imshow(img)
            ax.scatter(coords[0], coords[1], s=10, marker="o", color="crimson")
            figure1.canvas.draw()
        cid = figure1.canvas.mpl_connect('button_press_event', add_point)
        plt.show()

    #  non-interactive, works better if there is an intense ring
    # TODO: Make method more robust with respect to obviously wrong points
    else:
        coords = get_max_positions(img, num_points=num_points,radius=suspected_radius)
    a = fit_ellipse(np.array(coords[0]), np.array(coords[1]))
    center = ellipse_center(a)  # (x,y)
    lengths = ellipse_axis_length(a)
    angle = ellipse_angle_of_rotation(a)
    print("The center is:", center)
    print("The major and minor axis lengths are:", lengths)
    print("The angle of rotation is:", angle)
    if plot:
        print("plotting")
        plt.scatter(coords[0], coords[1])
        ellipse = Ellipse((center[1], center[0]), lengths[0] * 2, lengths[1] * 2, angle=angle, fill=False)
        fig = plt.figure()
        axe = fig.add_subplot(111)
        axe.imshow(img)
        axe.add_patch(ellipse)
        plt.show()
    return center, lengths, angle


def advanced_solve_ellipse(img, center, lengths, angle, phase_width, radius, num_points=500):
    """ This is a method in development. Better at optimizing angular correlations

    This method is in development. Due to the fact that an ellipse will have maximum 2-fold symmetry when the center
    is correctly determined and maximum 2*n-fold symmetry when the major/ minor axes as well as the angle of rotation
    is correct. This algorithm optimizes for these quantities.

    Parameters:
    -----------
    img: Array-like
        2-d Array of some image
    range: list
        The lower and upper limits to look at. Usually should just look at the first ring.
    num_points: int
        The number of points to look at to determine the characteristic ellipse.

    Return:
    -------------
    center: list
        The x and y coordinates of the center
    lengths: list
        The major and minor axes
    angle: float
        The angle of rotation for the ellipse
    """

    # Brute force testing method...

    x = np.linspace(-5,5,20)
    y = np.linspace(-5, 5, 20)

    pol =[[convert(img, angle=None, lengths=None, center=np.add(center[x1,y1]), phase_width=phase_width, radius=radius)
           for x1 in x]for y1 in y]
    


def invcot(val):
    return (np.pi/2) - np.arctan(val)


def get_max_positions(image, num_points=None, radius=None):
    i_shape = np.shape(image)
    flattened_array = image.flatten()
    indexes = np.argsort(flattened_array)

    if isinstance(flattened_array, np.ma.masked_array):
        indexes = indexes[flattened_array.mask[indexes] == False]
    if radius is not None:
        center = [np.floor_divide(np.mean(indexes[-num_points:]), i_shape[1]),
                  np.remainder(np.mean(indexes[-num_points:]), i_shape[1])]
        print(center)
    # take top 5000 points make sure exclude zero beam
    cords = [np.floor_divide(indexes[-num_points:], i_shape[1]),
             np.remainder(indexes[-num_points:], i_shape[1])]  # [x axis (row),y axis (col)]
    return cords


import numpy as np

from hyperspy.signals import Signal2D
from hyperspy.misc.slicing import SpecialSlicers
from hyperspy._signals.lazy import LazySignal


class EMSignal(Signal2D):
    """
    The Diffraction Signal class extends the Hyperspy 2d signal class
    """
    _signal_type = "em_signal"

    def __init__(self, *args, **kwargs):
        """Basic unit of data in the program.

        Spectrums can be any dimension of array but all of the data should be of the
        same type from the same area... (Maybe define this better later... You can use it kind of however you want though)
        Extends the Signal2D Class from hyperspy so there is that added functionality

        Parameters
        ----------
        data : numpy array
           The signal data. It can be an array of any dimensions.
        axes : dictionary (optional)
            Dictionary to define the axes (see the
            documentation of the AxesManager class for more details).
        attributes : dictionary (optional)
            A dictionary whose items are stored as attributes.
        metadata : dictionary (optional)
            A dictionary containing a set of parameters
            that will to stores in the `metadata` attribute.
            Some parameters might be mandatory in some cases.
        original_metadata : dictionary (optional)
            A dictionary containing a set of parameters
            that will to stores in the `original_metadata` attribute. It
            typically contains all the parameters that has been
            imported from the original data file.
        Notes: For more parameters see hyperspy's Signal2D Class
        """
        Signal2D.__init__(self, *args, **kwargs)
        self.manav = MaskSlicer(self, isNavigation=True)
        self.masig = MaskSlicer(self, isNavigation=False)
        self.mask_passer= None

    def as_lazy(self, *args, **kwargs):
        res = super().as_lazy(*args, **kwargs)
        res.__class__ = LazyEMSignal
        res.__init__(**res._to_dictionary())
        return res

    def add_haadf_intensities(self, intensity_array, slope, intercept):
        """Add High Angle Annular Dark Field intensities for each point.

        Parameters
        -----------
        intensity_array: nd array
            An intensity array which is the same size of the navigation axis.  Acts as a measure of the thickness if
            there is a calculated normalized intensity. For masking in real space.  Matches data input NOT the real
            space coordinates from Hyperspy. (To match Hyperspy use np.transpose on intensity array)
        """
        if not self.metadata.has_item('HAADF'):
            self.metadata.add_node('HAADF.intensity')
            self.metadata.add_node('HAADF.filter_slope')
            self.metadata.add_node('HAADF.filter_intercept')
        if self.axes_manager.navigation_shape != np.shape(np.transpose(intensity_array)):
            print("The navigation axes and intensity array don't match")
            return
        self.metadata.HAADF.intensity = np.transpose(intensity_array)
        self.metadata.HAADF.filter_slope = slope
        self.metadata.HAADF.filter_intercept = intercept
        return

    def get_thicknesses(self):
        return self.metadata.HAADF.intensity*self.metadata.HAADF.filter_slope+self.metadata.HAADF.filter_intercept

    def thickness_filter(self):
        """Filter based on HAADF intensities

        Returns
        ------------------
        th_filter: array-like
            Integers which are used to filter into different thicknesses. Basically used to bin the
            signal
        thicknesses: 1-d array
            The thicknesses for the signal at every integer.
        """
        thickness = self.get_thicknesses()
        twosigma = 2 * np.std(thickness)
        deviation = np.subtract(thickness, np.mean(thickness))
        th_filter = thickness
        th_filter[deviation > twosigma] = 0
        th_filter[deviation < twosigma] = 0
        th_filter[(-twosigma < deviation) & (deviation <= -twosigma/2)] = 1
        th_filter[(-twosigma / 2 < deviation) & (deviation <= 0)] = 2
        th_filter[(0 < deviation) & (deviation <= twosigma/2)] = 3
        th_filter[(twosigma / 2 < deviation) & (deviation <= twosigma)] = 4
        thickness = [np.mean(thickness) - 3*twosigma/2,
                     np.mean(thickness) - twosigma/2,
                     np.mean(thickness) +twosigma/2,
                     np.mean(thickness) + 3 * twosigma / 2]
        return th_filter, thickness

    def set_axes(self, index, name=None, scale=None, units=None, offset=None):
        """Set axes of the signal

        Parameters
        ----------
        index: int
           The index of the axes
        name: str
            The name of the axis
        scale : float
            The scale fo the axis
        units : str
            The units of the axis
        offset : float
            The offset of the axes
        """
        if name is not None:
            self.axes_manager[index].name = name
        if scale is not None:
            self.axes_manager[index].scale = scale
        if units is not None:
            self.axes_manager[index].units = units
        if offset is not None:
            self.axes_manager[index].offset = offset

    def mask_below(self, value, unmask=False):
        """Applies a mask to every pixel with an average value below value

        Parameters
        ----------
        value: float
            The maximum pixel value to apply a mask to.
        unmask: bool
            Unmask any pixel with a value below value
        """
        self.add_mask()
        print(value)
        print(self)
        print(np.less(self.data, value))
        print(self.data.mask)
        self.data.mask[self.data < value] = not unmask

    def mask_above(self, value, unmask=False):
        """Applies a mask to every pixel with a value below some value

        Parameters
        ----------
        value: float
            The minimum pixel value to apply a mask to.
        unmask: bool
            Unmask any pixel with a value above value
        """
        self.add_mask()
        print(self.data > value)
        self.data.mask[self.data > value] = not unmask

    def mask_where(self, condition):
        """Mask at some condition

        Parameters:
        -------------
        condition: array_like
            Masking condition

        """
        self.add_mask()
        self.data.mask[condition] = True
        return

    def mask_border(self, pixels=1):
        self.add_mask()
        if isinstance(pixels, int):
            pixels = (self.axes_manager.signal_axes[0].value2index(pixels))
        self.data.mask[..., -pixels:] = True
        self.data.mask[..., : pixels] = True
        self.data.mask[..., : pixels, :] = True
        self.data.mask[..., -pixels:, :] = True

    def add_mask(self):
        if not isinstance(self.data, np.ma.masked_array):
            self.data = np.ma.asarray(self.data)
            self.data.mask = False  # setting all values to unmasked

    def mask_circle(self, center, radius, unmask=False):
        # TODO: Add more shapes
        """Applies a mask to every pixel using a shape and the appropriate definition

        Parameters
        ----------
        shape: str
            Acceptable shapes ['rectangle, 'circle']
        data: list
            Define shapes. eg 'rectangle' -> [x1,x2,y1,y2] 'circle' -> [radius, x,y]
            data allows indexing with floats and the axes described for the signal
        unmask: bool
            Unmask any pixels in the defined shape
        """
        self.add_mask()
        if not all(isinstance(item, int) for item in center):
            center = (self.axes_manager.signal_axes[1].value2index(center[1]),
                     self.axes_manager.signal_axes[0].value2index(center[0]))
        if not isinstance(radius, int):
            radius = self.axes_manager.signal_axes[0].value2index(radius)
        x_ind, y_ind = np.meshgrid(range(-radius, radius + 1), range(-radius, radius + 1))
        r = np.sqrt(x_ind ** 2 + y_ind ** 2)
        inside = r < radius
        x_ind, y_ind = x_ind[inside]+int(center[0]), y_ind[inside]+int(center[1])
        self.data.mask[..., x_ind, y_ind] = True
        return

    def get_signal_axes_values(self):
        """ Returns the values for each pixel of the signal.  Useful for plotting without using hyperspy

        Returns
        ----------
        axis0: array-like
            The values for axis 0
        axis1: array-like
            The values for axis 1
        """
        axis0 = np.linspace(start=self.axes_manager.signal_axes[0].offset,
                            stop=(self.axes_manager.signal_axes[0].size *
                                  self.axes_manager.signal_axes[1].scale +
                                  self.axes_manager.signal_axes[0].offset),
                            num=self.axes_manager.signal_axes[0].size)
        axis1 = np.linspace(start=self.axes_manager.signal_axes[1].offset,
                            stop=(self.axes_manager.signal_axes[1].size *
                                  self.axes_manager.signal_axes[1].scale +
                                  self.axes_manager.signal_axes[1].offset),
                            num=self.axes_manager.signal_axes[1].size)
        return axis0, axis1


class MaskSlicer(SpecialSlicers):
    """
    Expansion of the Special Slicer class. Used for applying a mask
    """
    def __setitem__(self, key, value):
        if isinstance(self.obj, MaskPasser):
            array_slices = self.obj.signal._get_array_slices(key, self.isNavigation)
            if self.isNavigation == self.obj.isNavigation:
                print("You can't used masig or manav twice")
            self.obj.signal.add_mask()
            array_slices = tuple([slice1 if not (slice1 == slice(None, None, None)) else slice2 for
                                  slice1, slice2 in zip(self.obj.slice, array_slices)])
            self.obj.signal.data.mask[array_slices] = value
        else:
            array_slices = self.obj._get_array_slices(key, self.isNavigation)
            self.obj.add_mask()
            self.obj.data.mask[array_slices] = value

    def __getitem__(self, key, out=None):
        if isinstance(self.obj, MaskPasser):
            if self.isNavigation == self.obj.isNavigation:
                print("You can't used masig or manav twice")
                return
            array_slices = self.obj.signal._get_array_slices(key, self.isNavigation)
            array_slices = tuple([slice1 if not (slice1 == slice(None, None, None)) else slice2 for
                                  slice1, slice2 in zip(self.obj.slice, array_slices)])
            return MaskPasser(self.obj.signal, array_slices, self.isNavigation)
        else:
            array_slices = self.obj._get_array_slices(key, self.isNavigation)
            return MaskPasser(self.obj, array_slices, self.isNavigation)


class MaskPasser():
    def __init__(self, s, sl, nav):
        self.signal = s
        self.slice = sl
        self.isNavigation = nav
        self.manav = MaskSlicer(self, isNavigation=True)
        self.masig = MaskSlicer(self, isNavigation=False)

    def mask_below(self, maximum):
        """Mask below the max value

        Parameters:
        -------------
        maximum: float
            Mask any values in the slice below the maximum value
        """
        self.signal.add_mask()
        self.signal.data.mask[self.slice][(self.signal.data[self.slice] < maximum)] = True
        return

    def mask_above(self, minimum):
        """Mask above the minimum value

        Parameters:
        -------------
        minimum: float
            Mask any values in the slice above the minimum value
        """
        self.signal.add_mask()
        self.signal.data.mask[self.slice][(self.signal.data[self.slice] > minimum)] = True
        return

    def mask_where(self, condition):
        """Mask at some condition

        Parameters:
        -------------
        condition: array_like
            Masking condition

        """
        self.signal.add_mask()
        self.signal.data.mask[self.slice][condition] = True
        return

    def mask_circle(self, center, radius, unmask=False):
        # TODO: Add more shapes
        """Applies a mask to every pixel using a shape and the appropriate definition

        Parameters
        ----------
        center: tuple
            The (x,y) center of the circle
        radius: float or int
            The radius of the circle
        unmask: bool
            Unmask any pixels in the defined shape
        """
        self.signal.add_mask()
        if not all(isinstance(item, int) for item in center):
            center = (self.signal.axes_manager.signal_axes[1].value2index(center[1]),
                     self.signal.axes_manager.signal_axes[0].value2index(center[0]))
        if not isinstance(radius, int):
            radius = self.signal.axes_manager.signal_axes[0].value2index(radius)
        x_ind, y_ind = np.meshgrid(range(-radius, radius + 1), range(-radius, radius + 1))
        r = np.sqrt(x_ind ** 2 + y_ind ** 2)
        inside = r < radius
        x_ind, y_ind = x_ind[inside]+int(center[0]), y_ind[inside]+int(center[1])
        self.signal.data.mask[self.slice][..., x_ind, y_ind] = not unmask
        return


class LazyEMSignal(LazySignal,EMSignal):

    _lazy = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)



from unittest import TestCase
import matplotlib.pyplot as plt

from empyer.io import load


class TestPolarSignal(TestCase):
    def setUp(self):
        self.ps = load('/media/hdd/home/FEMImages_Angular_analyzed/20180415_ZCA-HH-_50W_3.8mT_RT_78s-0.22nm_sec_295/15.49.20 Spectrum image_pos01_angular.hdf5')

    def test_power_spectrum(self):
        power = self.ps.get_power_spectrum()
        power.isig[2:12, 3.0:6.0].plot()
        plt.show()

    def test_summed_power_spectrum(self):
        print(self.ps)
        sum_power = self.ps.get_summed_power_spectrum()
        sum_power.isig[2:12, 3.0:6.0].plot()
        plt.show()
        sum_power.isig[6, 3.0:8.0].plot()
        plt.show()

    def test_lazy(self):
        print(self.ps.as_lazy())


from unittest import TestCase
import numpy as np
from empyer.misc.cartesain_to_polar import convert
from empyer.misc.image import random_ellipse
from timeit import timeit


class TestConvert(TestCase):
    def setUp(self):
        self.d = np.zeros((512, 512))
        self.center = [276, 256]
        self.lengths = sorted(np.random.rand(2) * 100 + 100, reverse=True)
        self.angle = np.random.rand() * np.pi
        rand_points = random_ellipse(num_points=100, center=self.center, foci=self.lengths, angle=self.angle)

        self.d[rand_points[:, 0], rand_points[:, 1]] = 10

    def test_2d_convert(self):
        start = timeit()
        conversion = convert(self.d, center=self.center, angle=self.angle, lengths=self.lengths, phase_width=720)
        stop = timeit()
        print((stop-start))
        s = np.sum(conversion, axis=1)
        even = np.sum(conversion, axis=0)
        self.assertLess((s > max(s)/2).sum(), 4)






from setuptools import setup, find_packages
from os import path
from io import open

here = path.abspath(path.dirname(__file__))


def readme():
    with open('README.rst') as f:
        return f.read()


setup(name='empyer',
      version='0.225',
      description='Electron Microscopy tools for analyzing 4 and 5 dimensional STEM datasets',
      long_description=readme(),
      keywords='STEM Electron Microscopy Glass',
      url='https://github.com/CSSFrancis/empyer',
      author='CSSFrancis',
      author_email='csfrancis@wisc.edu',
      liscense='MIT',
      packages=['empyer',
                'empyer.misc',
                'empyer.signals',
                'empyer.simulate'],
      install_requires=['hyperspy >=1.5',
                        'numpy>=1.10,!=1.70.0',
                        'matplotlib',
                        'scipy'],
      #include_package_data=True,
      package_data={
          'empyer': ["hyperspy_extension.yaml"],
      },
      #  entry point is hyperspy.extensions, registering empyer as a
      entry_points={'hyperspy.extensions': ['empyer = empyer']},
      zip_safe=False)



from unittest import TestCase
import numpy as np
import matplotlib.pyplot as plt

from empyer.simulate.simulate_patterns import simulate_pattern, simulate_symmetry, cartesian_to_ellipse, random_pattern
from empyer.misc.ellipse_analysis import solve_ellipse
from empyer.misc.cartesain_to_polar import convert
from empyer.misc.angular_correlation import angular_correlation,power_spectrum


class TestSimulations(TestCase):
    def test_simulate_symmetry(self):
        sim = simulate_symmetry(symmetry=6, iterations=10000)
        plt.plot(sim[0])
        plt.show()
        power_sim = np.mean([np.fft.fft(s).real ** 2 for s in sim], axis=0)
        plt.plot(power_sim)
        plt.show()

    def test_simulation(self):
        random_pattern(4, 4)

    def test_simulate_image(self):
        i = simulate_pattern(4, k=100, num_clusters=3, probe_size=20, center=[256, 256], angle=0, lengths=[20, 10])
        c, a, lengths = solve_ellipse(i)

    def test_cartesian_to_ellipse(self):
        x, y = cartesian_to_ellipse(center=[256, 256], angle=np.pi /4, lengths=[1.5, 1])
        plt.scatter(x, y)
        plt.show()

    def test_simulation_correction(self):
        center = [263, 274]
        angle = np.pi/3
        lengths =[11,10]
        i = simulate_pattern(4,
                             k=100,
                             num_clusters=300,
                             probe_size=20,
                             center=center,
                             angle=angle,
                             lengths=lengths)
        i = np.random.poisson(i)
        c, l, a = solve_ellipse(i)
        # asserting the ellipse is correctly identified
        self.assertAlmostEqual(center[0], c[0], places=-1)
        self.assertAlmostEqual(center[1], c[1], places=-1)
        self.assertAlmostEqual(l[0]/l[1], max(lengths)/min(lengths), places=-1)
        self.assertAlmostEqual(angle, a, places=1)
        # converting to polar coordinates
        conversion = convert(img=i, center=c, lengths=l, angle=a)
        ac = angular_correlation(conversion, normalize=True)
        ps = power_spectrum(ac)
        #plt.imshow(i)
        #plt.show()
        plt.imshow(conversion[:, 0:359])
        plt.figure()
        plt.imshow(conversion[:, 360:719])
        plt.show()
        #plt.imshow(ac)
        plt.imshow(ps[:, 0:12])
        plt.show()





import unittest
from CusterOperations.seqtoHDF5 import loadHeader, loadMRCfile,saveFile


class MyTestCase(unittest.TestCase):
    def setUp(self):
        self.directory = '/media/hdd/home/FEM/DenseSampling/'
        self.filename = '16-24-03.211.seq'
        self.darkrefFile = '/media/hdd/home/FEM/DenseSampling/16-24-03.211.seq.dark.mrc'
        self.gainrefFile ='/media/hdd/home/FEM/DenseSampling/16-24-03.211.seq.gain.mrc'

    def test_read_header(self):
        filename ='/media/hdd/home/FEM/DenseSampling/16-47-57.638.seq'
        darkrefFile = '/media/hdd/home/FEM/DenseSampling/16-47-57.638.seq.dark.mrc'
        darkref = loadMRCfile(darkrefFile)
        nframes, head = loadHeader(filename, darkref)
        print(nframes, head)

    def test_save(self):
        dark = loadMRCfile(self.darkrefFile)
        gain = loadMRCfile(self.gainrefFile)
        numframes, true_imagesize = loadHeader(self.directory+self.filename, dark)
        saveFile(directory='/media/hdd/home/FEM/DenseSampling/',
                 filename=self.filename,
                 darkref=dark,
                 gainref=gain,
                 numframes=numframes,
                 true_imagesize=true_imagesize,
                 savename='test')

if __name__ == '__main__':
    unittest.main()
